#!/usr/bin/env python

# Copyright 2015 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from charms.reactive import when
from charms.reactive import hook, when_any
from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core.templating import render

from subprocess import Popen
from subprocess import PIPE
from subprocess import STDOUT
from subprocess import CalledProcessError

amslb_nginx_log_rotate = """/var/log/nginx.*.log {
    daily
    missingok
    rotate 14
    compress
    delaycompress
    notifempty
    create 0640 www-data adm
    sharedscripts
    prerotate
        if [ -d /etc/logrotate.d/httpd-prerotate ]; then \\
            run-parts /etc/logrotate.d/httpd-prerotate; \\
        fi \\
    endscript
    postrotate
        invoke-rc.d nginx rotate >/dev/null 2>&1
    endscript
}"""


@when('config.changed.port')
def close_old_port():
    config = hookenv.config()
    old_port = config.previous('port')
    if not old_port:
        return
    try:
        hookenv.close_port(old_port)
    except CalledProcessError:
        hookenv.log('Port %d already closed, skipping.' % old_port)


def maybe_write_amslb_logrotate_config():
    filename = '/etc/logrotate.d/amslb_nginx'
    if not os.path.exists(filename):
        # Set log rotation for amslb log file
        with open(filename, 'w+') as fp:
            fp.write(amslb_nginx_log_rotate)


def configure_site(site, template, **kwargs):
    """ configures vhost
    Arguments:
    site: Site name
    template: template to process in templates/<template.conf>
    **kwargs: additional dict items to append to template variables exposed
              through the site.toml
    """
    hookenv.status_set('maintenance', 'Configuring site {}'.format(site))

    config = hookenv.config()
    context = {}
    context['host'] = config['host']
    context['port'] = config['port']
    context.update(**kwargs)

    conf_path = '/etc/nginx/nginx.conf'
    if os.path.exists(conf_path):
        os.remove(conf_path)

    render(source=template,
           target=conf_path,
           context=context)

    host.service_reload('nginx')
    hookenv.status_set('active', '')


@when('nginx.available', 'endpoint.ams.available')
def install_load_balancer(ams):
    ''' Create the default vhost template for load balancing '''
    port = hookenv.config('port')
    hookenv.open_port(port)
    services = ams.services()
    configure_site(
            'amslb',
            'amslb.conf',
            server_name='_',
            services=services,
            port=port,
            proxy_timeout=hookenv.config('proxy_timeout')
    )

    maybe_write_amslb_logrotate_config()
    hookenv.status_set('active', 'Loadbalancer ready.')


@hook('upgrade-charm')
def upgrade_charm():
    maybe_write_amslb_logrotate_config()


@when_any('nginx.available', 'upgrade-charm')
def set_nginx_version():
    ''' Surface the currently deployed version of nginx to Juju '''
    cmd = 'nginx -v'
    p = Popen(cmd, shell=True,
              stdin=PIPE,
              stdout=PIPE,
              stderr=STDOUT,
              close_fds=True)
    raw = p.stdout.read()
    # The version comes back as:
    # nginx version: nginx/1.10.0 (Ubuntu)
    version = raw.split(b'/')[-1].split(b' ')[0]
    hookenv.application_version_set(version.rstrip())


@when('loadbalancer.available')
def provide_loadbalancing(loadbalancer):
    '''Send the public address and port to the public-address interface, so
    the subordinates can get the public address of this loadbalancer.'''
    loadbalancer.set_address_port(hookenv.unit_get('public-address'),
                                  hookenv.config('port'))
